package sun.java2d.cmm.lcms;

import sun.java2d.cmm.ProfileDeferralMgr;
import java.awt.image.SampleModel;
import java.awt.image.Raster;
import java.awt.color.ColorSpace;
import java.awt.image.ColorModel;
import java.awt.image.WritableRaster;
import java.awt.color.CMMException;
import java.awt.image.BufferedImage;
import java.awt.color.ICC_Profile;
import sun.java2d.cmm.ColorTransform;

public class LCMSTransform implements ColorTransform
{
    long ID;
    private int inFormatter;
    private boolean isInIntPacked;
    private int outFormatter;
    private boolean isOutIntPacked;
    ICC_Profile[] profiles;
    LCMSProfile[] lcmsProfiles;
    int renderType;
    int transformType;
    private int numInComponents;
    private int numOutComponents;
    private Object disposerReferent;
    
    public LCMSTransform(ICC_Profile icc_Profile, int n, int transformType) {
        this.inFormatter = 0;
        this.isInIntPacked = false;
        this.outFormatter = 0;
        this.isOutIntPacked = false;
        this.numInComponents = -1;
        this.numOutComponents = -1;
        this.disposerReferent = new Object();
        (this.profiles = new ICC_Profile[1])[0] = icc_Profile;
        (this.lcmsProfiles = new LCMSProfile[1])[0] = LCMS.getProfileID(icc_Profile);
        this.renderType = ((n == -1) ? 0 : n);
        this.transformType = transformType;
        this.numInComponents = this.profiles[0].getNumComponents();
        this.numOutComponents = this.profiles[this.profiles.length - 1].getNumComponents();
    }
    
    public LCMSTransform(ColorTransform[] array) {
        this.inFormatter = 0;
        this.isInIntPacked = false;
        this.outFormatter = 0;
        this.isOutIntPacked = false;
        this.numInComponents = -1;
        this.numOutComponents = -1;
        this.disposerReferent = new Object();
        int n = 0;
        for (int i = 0; i < array.length; ++i) {
            n += ((LCMSTransform)array[i]).profiles.length;
        }
        this.profiles = new ICC_Profile[n];
        this.lcmsProfiles = new LCMSProfile[n];
        LCMSTransform lcmsTransform;
        for (int n2 = 0, j = 0; j < array.length; n2 += lcmsTransform.profiles.length, ++j) {
            lcmsTransform = (LCMSTransform)array[j];
            System.arraycopy(lcmsTransform.profiles, 0, this.profiles, n2, lcmsTransform.profiles.length);
            System.arraycopy(lcmsTransform.lcmsProfiles, 0, this.lcmsProfiles, n2, lcmsTransform.lcmsProfiles.length);
        }
        this.renderType = ((LCMSTransform)array[0]).renderType;
        this.numInComponents = this.profiles[0].getNumComponents();
        this.numOutComponents = this.profiles[this.profiles.length - 1].getNumComponents();
    }
    
    @Override
    public int getNumInComponents() {
        return this.numInComponents;
    }
    
    @Override
    public int getNumOutComponents() {
        return this.numOutComponents;
    }
    
    private synchronized void doTransform(LCMSImageLayout lcmsImageLayout, LCMSImageLayout lcmsImageLayout2) {
        if (this.ID == 0L || this.inFormatter != lcmsImageLayout.pixelType || this.isInIntPacked != lcmsImageLayout.isIntPacked || this.outFormatter != lcmsImageLayout2.pixelType || this.isOutIntPacked != lcmsImageLayout2.isIntPacked) {
            if (this.ID != 0L) {
                this.disposerReferent = new Object();
            }
            this.inFormatter = lcmsImageLayout.pixelType;
            this.isInIntPacked = lcmsImageLayout.isIntPacked;
            this.outFormatter = lcmsImageLayout2.pixelType;
            this.isOutIntPacked = lcmsImageLayout2.isIntPacked;
            this.ID = LCMS.createTransform(this.lcmsProfiles, this.renderType, this.inFormatter, this.isInIntPacked, this.outFormatter, this.isOutIntPacked, this.disposerReferent);
        }
        LCMS.colorConvert(this, lcmsImageLayout, lcmsImageLayout2);
    }
    
    @Override
    public void colorConvert(BufferedImage bufferedImage, BufferedImage bufferedImage2) {
        try {
            if (!bufferedImage2.getColorModel().hasAlpha()) {
                LCMSImageLayout imageLayout = LCMSImageLayout.createImageLayout(bufferedImage2);
                if (imageLayout != null) {
                    LCMSImageLayout imageLayout2 = LCMSImageLayout.createImageLayout(bufferedImage);
                    if (imageLayout2 != null) {
                        this.doTransform(imageLayout2, imageLayout);
                        return;
                    }
                }
            }
        }
        catch (LCMSImageLayout.ImageLayoutException ex) {
            throw new CMMException("Unable to convert images");
        }
        WritableRaster raster = bufferedImage.getRaster();
        WritableRaster raster2 = bufferedImage2.getRaster();
        ColorModel colorModel = bufferedImage.getColorModel();
        ColorModel colorModel2 = bufferedImage2.getColorModel();
        int width = bufferedImage.getWidth();
        int height = bufferedImage.getHeight();
        int numColorComponents = colorModel.getNumColorComponents();
        int numColorComponents2 = colorModel2.getNumColorComponents();
        int n = 8;
        float n2 = 255.0f;
        for (int i = 0; i < numColorComponents; ++i) {
            if (colorModel.getComponentSize(i) > 8) {
                n = 16;
                n2 = 65535.0f;
            }
        }
        for (int j = 0; j < numColorComponents2; ++j) {
            if (colorModel2.getComponentSize(j) > 8) {
                n = 16;
                n2 = 65535.0f;
            }
        }
        float[] array = new float[numColorComponents];
        float[] array2 = new float[numColorComponents];
        ColorSpace colorSpace = colorModel.getColorSpace();
        for (int k = 0; k < numColorComponents; ++k) {
            array[k] = colorSpace.getMinValue(k);
            array2[k] = n2 / (colorSpace.getMaxValue(k) - array[k]);
        }
        ColorSpace colorSpace2 = colorModel2.getColorSpace();
        float[] array3 = new float[numColorComponents2];
        float[] array4 = new float[numColorComponents2];
        for (int l = 0; l < numColorComponents2; ++l) {
            array3[l] = colorSpace2.getMinValue(l);
            array4[l] = (colorSpace2.getMaxValue(l) - array3[l]) / n2;
        }
        int hasAlpha = colorModel2.hasAlpha();
        int n3 = colorModel.hasAlpha() && hasAlpha;
        float[] array5;
        if (!(hasAlpha == 0)) {
            array5 = new float[numColorComponents2 + 1];
        }
        else {
            array5 = new float[numColorComponents2];
        }
        if (n == 8) {
            byte[] array9 = new byte[width * numColorComponents];
            byte[] array10 = new byte[width * numColorComponents2];
            float[] array11 = null;
            if (!(n3 == 0)) {
                array11 = new float[width];
            }
            Object o3;
            Object o4;
            try {
                o3 = new LCMSImageLayout(array9, array9.length / this.getNumInComponents(), LCMSImageLayout.CHANNELS_SH(this.getNumInComponents()) | LCMSImageLayout.BYTES_SH(1), this.getNumInComponents());
                o4 = new LCMSImageLayout(array10, array10.length / this.getNumOutComponents(), LCMSImageLayout.CHANNELS_SH(this.getNumOutComponents()) | LCMSImageLayout.BYTES_SH(1), this.getNumOutComponents());
            }
            catch (LCMSImageLayout.ImageLayoutException ex3) {
                throw new CMMException("Unable to convert images");
            }
            for (int n9 = 0; n9 < height; ++n9) {
                Object dataElements3 = null;
                float[] normalizedComponents2 = null;
                int n10 = 0;
                for (int x3 = 0; x3 < width; ++x3) {
                    dataElements3 = raster.getDataElements(x3, n9, dataElements3);
                    normalizedComponents2 = colorModel.getNormalizedComponents(dataElements3, normalizedComponents2, 0);
                    for (int n11 = 0; n11 < numColorComponents; ++n11) {
                        array9[n10++] = (byte)(int)((normalizedComponents2[n11] - array[n11]) * array2[n11] + 0.5f);
                    }
                    if (!(n3 == 0)) {
                        array11[x3] = normalizedComponents2[numColorComponents];
                    }
                }
                this.doTransform(o3, o4);
                Object dataElements4 = null;
                int n12 = 0;
                for (int x4 = 0; x4 < width; ++x4) {
                    for (int n13 = 0; n13 < numColorComponents2; ++n13) {
                        array5[n13] = (float)(array10[n12++] & 0xFF) * array4[n13] + array3[n13];
                    }
                    if (!(n3 == 0)) {
                        array5[numColorComponents2] = array11[x4];
                    }
                    else if (!(hasAlpha == 0)) {
                        array5[numColorComponents2] = 1.0f;
                    }
                    dataElements4 = colorModel2.getDataElements(array5, 0, dataElements4);
                    raster2.setDataElements(x4, n9, dataElements4);
                }
            }
        }
        else {
            short[] array6 = new short[width * numColorComponents];
            short[] array7 = new short[width * numColorComponents2];
            float[] array8 = null;
            if (!(n3 == 0)) {
                array8 = new float[width];
            }
            Object o;
            Object o2;
            try {
                o = new LCMSImageLayout(array6, array6.length / this.getNumInComponents(), LCMSImageLayout.CHANNELS_SH(this.getNumInComponents()) | LCMSImageLayout.BYTES_SH(2), this.getNumInComponents() * 2);
                o2 = new LCMSImageLayout(array7, array7.length / this.getNumOutComponents(), LCMSImageLayout.CHANNELS_SH(this.getNumOutComponents()) | LCMSImageLayout.BYTES_SH(2), this.getNumOutComponents() * 2);
            }
            catch (LCMSImageLayout.ImageLayoutException ex2) {
                throw new CMMException("Unable to convert images");
            }
            for (int n4 = 0; n4 < height; ++n4) {
                Object dataElements = null;
                float[] normalizedComponents = null;
                int n5 = 0;
                for (int x = 0; x < width; ++x) {
                    dataElements = raster.getDataElements(x, n4, dataElements);
                    normalizedComponents = colorModel.getNormalizedComponents(dataElements, normalizedComponents, 0);
                    for (int n6 = 0; n6 < numColorComponents; ++n6) {
                        array6[n5++] = (short)(int)((normalizedComponents[n6] - array[n6]) * array2[n6] + 0.5f);
                    }
                    if (!(n3 == 0)) {
                        array8[x] = normalizedComponents[numColorComponents];
                    }
                }
                this.doTransform(o, o2);
                Object dataElements2 = null;
                int n7 = 0;
                for (int x2 = 0; x2 < width; ++x2) {
                    for (int n8 = 0; n8 < numColorComponents2; ++n8) {
                        array5[n8] = (float)(array7[n7++] & 0xFFFF) * array4[n8] + array3[n8];
                    }
                    if (!(n3 == 0)) {
                        array5[numColorComponents2] = array8[x2];
                    }
                    else if (!(hasAlpha == 0)) {
                        array5[numColorComponents2] = 1.0f;
                    }
                    dataElements2 = colorModel2.getDataElements(array5, 0, dataElements2);
                    raster2.setDataElements(x2, n4, dataElements2);
                }
            }
        }
    }
    
    @Override
    public void colorConvert(Raster raster, WritableRaster writableRaster, float[] array, float[] array2, float[] array3, float[] array4) {
        SampleModel sampleModel = raster.getSampleModel();
        SampleModel sampleModel2 = writableRaster.getSampleModel();
        int transferType = raster.getTransferType();
        int transferType2 = writableRaster.getTransferType();
        int n = transferType == 4 || transferType == 5;
        int n2 = transferType2 == 4 || transferType2 == 5;
        int width = raster.getWidth();
        int height = raster.getHeight();
        int numBands = raster.getNumBands();
        int numBands2 = writableRaster.getNumBands();
        float[] array5 = new float[numBands];
        float[] array6 = new float[numBands2];
        float[] array7 = new float[numBands];
        float[] array8 = new float[numBands2];
        for (int i = 0; i < numBands; ++i) {
            if (!(n == 0)) {
                array5[i] = 65535.0f / (array2[i] - array[i]);
                array7[i] = array[i];
            }
            else {
                if (transferType == 2) {
                    array5[i] = 2.0000305f;
                }
                else {
                    array5[i] = 65535.0f / (float)((1 << sampleModel.getSampleSize(i)) - 1);
                }
                array7[i] = 0.0f;
            }
        }
        for (int j = 0; j < numBands2; ++j) {
            if (!(n2 == 0)) {
                array6[j] = (array4[j] - array3[j]) / 65535.0f;
                array8[j] = array3[j];
            }
            else {
                if (transferType2 == 2) {
                    array6[j] = 0.49999237f;
                }
                else {
                    array6[j] = (float)((1 << sampleModel2.getSampleSize(j)) - 1) / 65535.0f;
                }
                array8[j] = 0.0f;
            }
        }
        int minY = raster.getMinY();
        int minY2 = writableRaster.getMinY();
        short[] array9 = new short[width * numBands];
        short[] array10 = new short[width * numBands2];
        Object o;
        Object o2;
        try {
            o = new LCMSImageLayout(array9, array9.length / this.getNumInComponents(), LCMSImageLayout.CHANNELS_SH(this.getNumInComponents()) | LCMSImageLayout.BYTES_SH(2), this.getNumInComponents() * 2);
            o2 = new LCMSImageLayout(array10, array10.length / this.getNumOutComponents(), LCMSImageLayout.CHANNELS_SH(this.getNumOutComponents()) | LCMSImageLayout.BYTES_SH(2), this.getNumOutComponents() * 2);
        }
        catch (LCMSImageLayout.ImageLayoutException ex) {
            throw new CMMException("Unable to convert rasters");
        }
        for (int k = 0; k < height; ++k, ++minY, ++minY2) {
            int minX = raster.getMinX();
            int n3 = 0;
            for (int l = 0; l < width; ++l, ++minX) {
                for (int b = 0; b < numBands; ++b) {
                    array9[n3++] = (short)(int)((raster.getSampleFloat(minX, minY, b) - array7[b]) * array5[b] + 0.5f);
                }
            }
            this.doTransform(o, o2);
            int minX2 = writableRaster.getMinX();
            int n4 = 0;
            for (int n5 = 0; n5 < width; ++n5, ++minX2) {
                for (int b2 = 0; b2 < numBands2; ++b2) {
                    writableRaster.setSample(minX2, minY2, b2, (float)(array10[n4++] & 0xFFFF) * array6[b2] + array8[b2]);
                }
            }
        }
    }
    
    @Override
    public void colorConvert(Raster raster, WritableRaster writableRaster) {
        LCMSImageLayout imageLayout = LCMSImageLayout.createImageLayout(writableRaster);
        if (imageLayout != null) {
            LCMSImageLayout imageLayout2 = LCMSImageLayout.createImageLayout(raster);
            if (imageLayout2 != null) {
                this.doTransform(imageLayout2, imageLayout);
                return;
            }
        }
        SampleModel sampleModel = raster.getSampleModel();
        SampleModel sampleModel2 = writableRaster.getSampleModel();
        int transferType = raster.getTransferType();
        int transferType2 = writableRaster.getTransferType();
        int width = raster.getWidth();
        int height = raster.getHeight();
        int numBands = raster.getNumBands();
        int numBands2 = writableRaster.getNumBands();
        int n = 8;
        float n2 = 255.0f;
        for (int i = 0; i < numBands; ++i) {
            if (sampleModel.getSampleSize(i) > 8) {
                n = 16;
                n2 = 65535.0f;
            }
        }
        for (int j = 0; j < numBands2; ++j) {
            if (sampleModel2.getSampleSize(j) > 8) {
                n = 16;
                n2 = 65535.0f;
            }
        }
        float[] array = new float[numBands];
        float[] array2 = new float[numBands2];
        for (int k = 0; k < numBands; ++k) {
            if (transferType == 2) {
                array[k] = n2 / 32767.0f;
            }
            else {
                array[k] = n2 / (float)((1 << sampleModel.getSampleSize(k)) - 1);
            }
        }
        for (int l = 0; l < numBands2; ++l) {
            if (transferType2 == 2) {
                array2[l] = 32767.0f / n2;
            }
            else {
                array2[l] = (float)((1 << sampleModel2.getSampleSize(l)) - 1) / n2;
            }
        }
        int minY = raster.getMinY();
        int minY2 = writableRaster.getMinY();
        if (n == 8) {
            byte[] array5 = new byte[width * numBands];
            byte[] array6 = new byte[width * numBands2];
            Object o3;
            Object o4;
            try {
                o3 = new LCMSImageLayout(array5, array5.length / this.getNumInComponents(), LCMSImageLayout.CHANNELS_SH(this.getNumInComponents()) | LCMSImageLayout.BYTES_SH(1), this.getNumInComponents());
                o4 = new LCMSImageLayout(array6, array6.length / this.getNumOutComponents(), LCMSImageLayout.CHANNELS_SH(this.getNumOutComponents()) | LCMSImageLayout.BYTES_SH(1), this.getNumOutComponents());
            }
            catch (LCMSImageLayout.ImageLayoutException ex2) {
                throw new CMMException("Unable to convert rasters");
            }
            for (int n8 = 0; n8 < height; ++n8, ++minY, ++minY2) {
                int minX3 = raster.getMinX();
                int n9 = 0;
                for (int n10 = 0; n10 < width; ++n10, ++minX3) {
                    for (int b3 = 0; b3 < numBands; ++b3) {
                        array5[n9++] = (byte)(int)((float)raster.getSample(minX3, minY, b3) * array[b3] + 0.5f);
                    }
                }
                this.doTransform(o3, o4);
                int minX4 = writableRaster.getMinX();
                int n11 = 0;
                for (int n12 = 0; n12 < width; ++n12, ++minX4) {
                    for (int b4 = 0; b4 < numBands2; ++b4) {
                        writableRaster.setSample(minX4, minY2, b4, (int)((float)(array6[n11++] & 0xFF) * array2[b4] + 0.5f));
                    }
                }
            }
        }
        else {
            short[] array3 = new short[width * numBands];
            short[] array4 = new short[width * numBands2];
            Object o;
            Object o2;
            try {
                o = new LCMSImageLayout(array3, array3.length / this.getNumInComponents(), LCMSImageLayout.CHANNELS_SH(this.getNumInComponents()) | LCMSImageLayout.BYTES_SH(2), this.getNumInComponents() * 2);
                o2 = new LCMSImageLayout(array4, array4.length / this.getNumOutComponents(), LCMSImageLayout.CHANNELS_SH(this.getNumOutComponents()) | LCMSImageLayout.BYTES_SH(2), this.getNumOutComponents() * 2);
            }
            catch (LCMSImageLayout.ImageLayoutException ex) {
                throw new CMMException("Unable to convert rasters");
            }
            for (int n3 = 0; n3 < height; ++n3, ++minY, ++minY2) {
                int minX = raster.getMinX();
                int n4 = 0;
                for (int n5 = 0; n5 < width; ++n5, ++minX) {
                    for (int b = 0; b < numBands; ++b) {
                        array3[n4++] = (short)(int)((float)raster.getSample(minX, minY, b) * array[b] + 0.5f);
                    }
                }
                this.doTransform(o, o2);
                int minX2 = writableRaster.getMinX();
                int n6 = 0;
                for (int n7 = 0; n7 < width; ++n7, ++minX2) {
                    for (int b2 = 0; b2 < numBands2; ++b2) {
                        writableRaster.setSample(minX2, minY2, b2, (int)((float)(array4[n6++] & 0xFFFF) * array2[b2] + 0.5f));
                    }
                }
            }
        }
    }
    
    @Override
    public short[] colorConvert(short[] array, short[] array2) {
        if (array2 == null) {
            array2 = new short[array.length / this.getNumInComponents() * this.getNumOutComponents()];
        }
        try {
            this.doTransform(new LCMSImageLayout(array, array.length / this.getNumInComponents(), LCMSImageLayout.CHANNELS_SH(this.getNumInComponents()) | LCMSImageLayout.BYTES_SH(2), this.getNumInComponents() * 2), new LCMSImageLayout(array2, array2.length / this.getNumOutComponents(), LCMSImageLayout.CHANNELS_SH(this.getNumOutComponents()) | LCMSImageLayout.BYTES_SH(2), this.getNumOutComponents() * 2));
            return array2;
        }
        catch (LCMSImageLayout.ImageLayoutException ex) {
            throw new CMMException("Unable to convert data");
        }
    }
    
    @Override
    public byte[] colorConvert(byte[] array, byte[] array2) {
        if (array2 == null) {
            array2 = new byte[array.length / this.getNumInComponents() * this.getNumOutComponents()];
        }
        try {
            this.doTransform(new LCMSImageLayout(array, array.length / this.getNumInComponents(), LCMSImageLayout.CHANNELS_SH(this.getNumInComponents()) | LCMSImageLayout.BYTES_SH(1), this.getNumInComponents()), new LCMSImageLayout(array2, array2.length / this.getNumOutComponents(), LCMSImageLayout.CHANNELS_SH(this.getNumOutComponents()) | LCMSImageLayout.BYTES_SH(1), this.getNumOutComponents()));
            return array2;
        }
        catch (LCMSImageLayout.ImageLayoutException ex) {
            throw new CMMException("Unable to convert data");
        }
    }
    
    static {
        if (ProfileDeferralMgr.deferring) {
            ProfileDeferralMgr.activateProfiles();
        }
    }
}
